/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * Contributor(s): Billy <bllaw@mmu.edu.my> http://linkvisitor.mozdev.org/
 *
 *
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL"), in which case the provisions of the GPL are applicable
 * instead of those above.  If you wish to allow use of your
 * version of this file only under the terms of the GPL and not to
 * allow others to use your version of this file under the MPL,
 * indicate your decision by deleting the provisions above and
 * replace them with the notice and other provisions required by
 * the GPL.  If you do not delete the provisions above, a recipient
 * may use your version of this file under either the MPL or the
 * GPL.
 */

var vlGlobalHistory = Components.classes['@mozilla.org/browser/global-history;1'].getService(Components.interfaces.nsIGlobalHistory);
var vlBrowserHistory = Components.classes["@mozilla.org/browser/global-history;1"].getService(Components.interfaces.nsIBrowserHistory);

// This is the node user right clicks on, null if it is not an address node.
// It is initiliased and reassigned in vlIsOverLink() only.
var vlOverLink = null;

// Modified from optimoz mosgestoverlay.js
// Given a node, try to get the address node of itself or its parent(s).
// Returns null if cannot find any.
function vlFindNodeLink(domPosition) {
  if (domPosition == document)
    return false;
  try {
    if(domPosition.nodeName == "A")
      return {node : domPosition, uri : domPosition.href};
    else
      return vlFindNodeLink(domPosition.parentNode);
  }
  catch(e) {
    return null;
  }
}

// Open the History Window in Mozilla and History Sidebar in Mozilla Firebird
function vlShowHistoryList() {
  try {
    toHistory(); // open the History Window in Mozilla 
  }
  catch(e) {
    toggleSidebar("viewHistorySidebar"); // should be in Mozilla Firebird
  }
}

// Returns true if the popup node is an address node.
function vlIsOverLink() {
  vlOverLink = vlFindNodeLink(document.popupNode);
  return (vlOverLink != null);
}

// Idea taken from Tabbrowser Extensions globaloverlay.js
// Once we added/removed the url to the history list,
// doing this will change the address node's colour.
// Unfortunately, it will only affect the node, and not other
// nodes with the same url in the page.
function vlRefreshLinkColor(aNode) {
	var oldHref = aNode.href;
	aNode.href = '';
	aNode.href = oldHref;
}

function vlOnPopupShown() {
  var selectedText = String(vlGetSelection());
  var textSelected = selectedText.length > 0;
  var isOverLink = vlIsOverLink();

  document.getElementById('vlToggleLink').setAttribute('disabled', (textSelected || !isOverLink));
  if (!textSelected && isOverLink) {
    var shownUrl = vlGetURLShownInPage(vlOverLink.uri);
    var visited = vlGlobalHistory.isVisited(shownUrl);
		var bundle = document.getElementById("vlStringBundle");
		if (visited)
      document.getElementById('vlToggleLink').label = bundle.getString('vlToggleLinkVisited');
    else
      document.getElementById('vlToggleLink').label = bundle.getString('vlToggleLinkUnvisited');
  }

  var linkSelected =
    textSelected &&
    ("links" in document.commandDispatcher.focusedWindow.document) &&
		(document.commandDispatcher.focusedWindow.document.links.length > 0);
  document.getElementById('vlVisitAll').setAttribute('disabled', !linkSelected);
  document.getElementById('vlUnvisitAll').setAttribute('disabled', !linkSelected);

  return;
}

// Modified from linky linky.js
function vlGetSelection() {
	var focusedWindow = document.commandDispatcher.focusedWindow;
	return focusedWindow.__proto__.getSelection.call(focusedWindow);
}

function vlToggleLink() {
  if (vlOverLink == null)
    return;

  var shownUrl = vlGetURLShownInPage(vlOverLink.uri);  
  var visited = vlGlobalHistory.isVisited(shownUrl);
//  alert('visited: ' + visited);
  if (visited)
    vlBrowserHistory.removePage(shownUrl);
  else
    vlGlobalHistory.addPage(shownUrl);

  vlRefreshLinkColor(vlOverLink.node);
  return;
}

// If there is only a host name or IP in the url with these protocols,
// mozilla browser will show the url with a '/' appended.
// ie., http://www.yahoo.com shown as http://www.yahoo.com/
const vlSpecialProtocolTest = /http|ftp|https|gopher|news|nntp|mailbox|snews|resource/i;
const vlHostOnlyURLTest = /^(\w+):\/\/[^\/]+$/;
function vlGetURLShownInPage(url) {
  var found = vlHostOnlyURLTest.exec(url);
  if (found) {
    var special = vlSpecialProtocolTest.test(found[1]);
    if (special) 
      url = url + '/';
  }
  return url;
}

// Modified from linky linky.js
function vlMarkSelectedLinks(wantVisited) {
  var selection = vlGetSelection();

  if (String(selection).length == 0 ||
      !("links" in document.commandDispatcher.focusedWindow.document))
    return;

  var links = document.commandDispatcher.focusedWindow.document.links;
  if (links.length == 0)
    return;

// not very efficient, but it does the work all right.
  for (var i = 0; i < links.length; i++) {
    if (selection.containsNode(links[i], true)) {
      var shownUrl = vlGetURLShownInPage(links[i].href);  
      if (wantVisited)
        vlGlobalHistory.addPage(shownUrl);
      else
        vlBrowserHistory.removePage(shownUrl);
      vlRefreshLinkColor(links[i]);
    }
  }
}


/*

var testRDF =
  Components.classes["@mozilla.org/rdf/rdf-service;1"].
  getService(Components.interfaces.nsIRDFService);

var testhistory =
  Components.classes["@mozilla.org/rdf/datasource;1?name=history"].
  getService(Components.interfaces.nsIRDFDataSource);

var testobserver = {
  onAssert : function(ds, s, p, t) {
    // added to the history
    var sValue = s.Value;
    var pValue = p.Value;
    var tValue = t.Value;
    if (s == testRDF.GetResource("NC:HistoryRoot") &&
        p == testRDF.GetResource("http://home.netscape.com/NC-rdf#child")) {
      var temp = t.Value;
    }
  },

  onUnassert : function(ds, s, p, t) {
    // removed from the history
    var sValue = s.Value;
    var pValue = p.Value;
    var tValue = t.Value;
    if (s == testRDF.GetResource("NC:HistoryRoot") &&
        p == testRDF.GetResource("http://home.netscape.com/NC-rdf#child")) {
      var temp = t.Value;
    }
  },

  onChange : function(ds, s, p, oldt, newt) {
    // Visit date updated, etc. 
  },

  onMove : function(ds, olds, news, p, t) {},
  beginUpdateBatch : function(ds) {},
  endUpdateBatch : function(ds) {}
};

  testhistory.AddObserver(testobserver);


  // to see all supported interfaces in mozilla
  var all = Components.interfaces;
  var doc = document.commandDispatcher.focusedWindow.open("about:blank").document;
  for (var i in all) {  
    doc.writeln('<p>' + i.toString() + '</p>');
  }
  doc.close();
  return;
*/
